@@ -11,13 +11,13 @@
  * @brief  Implementation of grammar relatedd module commands.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 2002-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2002-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2002-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -156,7 +156,10 @@ japi_change_grammar(int sd, char *prefixpath)
   /* file existing check */
   if (check_grammar_path(prefixpath) < 0) return;
   /* send data */
-  do_send(sd, "CHANGEGRAM\n");
+  /* do_send(sd, "CHANGEGRAM\n"); */
+  /* send data with its name */
+  snprintf(buf, MAXLINELEN, "CHANGEGRAM %s\n", prefixpath);
+  do_send(sd, buf);
   send_grammar(sd, prefixpath);
 }
 
@@ -181,7 +184,10 @@ japi_add_grammar(int sd, char *prefixpath)
   /* file existing check */
   if (check_grammar_path(prefixpath) < 0) return;
   /* send data */
-  do_send(sd, "ADDGRAM\n");
+  /* do_send(sd, "ADDGRAM\n"); */
+  /* send data with its name */
+  snprintf(buf, MAXLINELEN, "ADDGRAM %s\n", prefixpath);
+  do_send(sd, buf);
   send_grammar(sd, prefixpath);
 }
 
