@@ -45,13 +45,13 @@
  * Julius.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.6 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -98,23 +98,13 @@ bt_prepare(BACKTRELLIS *bt)
   TRELLIS_ATOM *tre, *tmp;
 
   /* free previously allocated data */
-  if (bt->root != NULL) free(bt->root);
-  if (bt->rw != NULL) free(bt->rw);
-  if (bt->num != NULL) free(bt->num);
+  mybfree2(&(bt->root));
+
+  /* reset entry point */
   bt->num = NULL;
   bt->rw = NULL;
-  bt->root = NULL;
-
-  /* free actual trellis atoms previously allocated */
-  tre = bt->list;
-  while(tre) {
-    tmp = tre->next;
-    free(tre);
-    tre = tmp;
-  }
-  /* reset entry point */
   bt->list = NULL;
-
+  bt->root = NULL;
 }  
 
 
@@ -144,6 +134,7 @@ void
 bt_store(BACKTRELLIS *bt, TRELLIS_ATOM *tatom)
 {
 #ifdef WORD_GRAPH
+  tatom->within_context = FALSE;
   tatom->within_wordgraph = FALSE;
 #endif
   tatom->next = bt->list;
@@ -172,13 +163,14 @@ bt_relocate_rw(BACKTRELLIS *bt)
   TRELLIS_ATOM *tre;
   int t;
   int totalnum, n;
+  TRELLIS_ATOM **tmp;
 
   if (bt->framelen == 0) {
     bt->num = NULL;
     return;
   }
 
-  bt->num = (int *)mymalloc(sizeof(int) * bt->framelen);
+  bt->num = (int *)mybmalloc2(sizeof(int) * bt->framelen, &(bt->root));
 
   /* count number of trellis atom (= survived word end) for each frame */
   for (t=0;t<bt->framelen;t++) bt->num[t] = 0;
@@ -199,12 +191,12 @@ bt_relocate_rw(BACKTRELLIS *bt)
   }
   
   /* allocate area */
-  bt->root = (TRELLIS_ATOM **)mymalloc(sizeof(TRELLIS_ATOM *) * totalnum);
-  bt->rw  = (TRELLIS_ATOM ***)mymalloc(sizeof(TRELLIS_ATOM **) * bt->framelen);
+  bt->rw  = (TRELLIS_ATOM ***)mybmalloc2(sizeof(TRELLIS_ATOM **) * bt->framelen, &(bt->root));
+  tmp = (TRELLIS_ATOM **)mybmalloc2(sizeof(TRELLIS_ATOM *) * totalnum, &(bt->root));
   n = 0;
   for (t=0;t<bt->framelen;t++) {
     if (bt->num[t] > 0) {
-      bt->rw[t] = (TRELLIS_ATOM **)&(bt->root[n]);
+      bt->rw[t] = (TRELLIS_ATOM **)&(tmp[n]);
       n += bt->num[t];
     }
   }
@@ -455,8 +447,17 @@ bt_binsearch_atom(BACKTRELLIS *bt, int t, WORD_ID wkey)
     /* same word with different context will be found:
        most likely one will be returned */
     maxscore = LOG_ZERO;
+    maxtre = NULL;
     i = left;
-    while (i >= 0 && (tmp = bt->rw[t][i])->wid == wkey) {
+    while (i >= 0) {
+      tmp = bt->rw[t][i];
+      if (tmp->wid != wkey) break;
+#ifdef WORD_GRAPH
+      /* only words on a graph path should be counted */
+      if (!tmp->within_wordgraph) {
+	i--; continue;
+      }
+#endif
       if (maxscore < tmp->backscore) {
 	maxscore = tmp->backscore;
 	maxtre = tmp;
@@ -464,16 +465,30 @@ bt_binsearch_atom(BACKTRELLIS *bt, int t, WORD_ID wkey)
       i--;
     }
     i = left;
-    while (i < bt->num[t] && (tmp = bt->rw[t][i])->wid == wkey) {
+    while (i < bt->num[t]) {
+      tmp = bt->rw[t][i];
+      if (tmp->wid != wkey) break;
+#ifdef WORD_GRAPH
+      /* only words on a graph path should be counted */
+      if (!tmp->within_wordgraph) {
+	i++; continue;
+      }
+#endif
       if (maxscore < tmp->backscore) {
 	maxscore = tmp->backscore;
 	maxtre = tmp;
       }
       i++;
     }
-
     tmp = maxtre;
-#endif /* WORD_GRAPH */
+#else
+#ifdef WORD_GRAPH
+    /* treat only words on a graph path */
+    if (! tmp->within_wordgraph) {
+      return NULL;
+    }
+#endif
+#endif /* WPAIR */
 
     return(tmp);
   } else {
