@@ -11,13 +11,13 @@
  * @brief  Output all information of recognition system to standard out.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.11 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -106,30 +106,34 @@ print_info()
 
   /* for backward compatibility with scoring tool (IPA99)... :-( */
   if (speech_input == SP_RAWFILE) {
-    j_printf("Speech input source: file\n");
+    j_printf("Speech input source: file\n\n");
   } else if (speech_input == SP_MFCFILE) {
-    j_printf("Speech input source: MFCC parameter file (HTK format)\n");
+    j_printf("Speech input source: MFCC parameter file (HTK format)\n\n");
   }
 
   if (speech_input != SP_MFCFILE) {
-    j_printf("Acoustic analysis condition:\n");
-    j_printf("\t   sample period = %4d ns      (%5d Hz)\n", smpPeriod, smpFreq);
-    j_printf("\t     window size = %4d samples (%.1f ms)\n", fsize,
-	     (float)smpPeriod * (float)fsize / 10000.0);
-    j_printf("\t     frame shift = %4d samples (%.1f ms)\n", fshift,
-	     (float)smpPeriod * (float)fshift / 10000.0);
-    j_printf("\t    delta window = %4d frames  (%.1f ms) around\n", delwin,
-	     (float)delwin * (float)smpPeriod * (float)fshift / 10000.0);
-    j_printf("\t        hi freq. = ");
-    if (hipass < 0) j_printf("OFF\n"); 
-    else j_printf("%5d Hz\n", hipass);
-    j_printf("\t        lo freq. = ");
-    if (lopass < 0) j_printf("OFF\n"); 
-    else j_printf("%5d Hz\n", lopass);
-    j_printf("\t  energy measure = ");
-    if (c0_required) j_printf("(_0) 0'th cepstral\n");
-    else j_printf("(_E) log energy\n");
 
+    put_para(&para);
+
+    j_printf("\t base setup from =");
+    if (para_htk.loaded == 1 || para_hmm.loaded == 1) {
+      if (para_hmm.loaded == 1) {
+	j_printf(" binhmm-embedded");
+	if (para_htk.loaded == 1) {
+	  j_printf(", then overridden by HTK Config and defaults");
+	}
+      } else {
+	if (para_htk.loaded == 1) {
+	  j_printf(" HTK Config (and HTK defaults)");
+	}
+      }
+    } else {
+      j_printf(" Julius defaults");
+    }
+    j_printf("\n");
+
+
+    j_printf("\n");
     j_printf("    spectral subtraction = ");
     if (ssload_filename || sscalc) {
       if (sscalc) {
@@ -142,8 +146,8 @@ print_info()
 	j_printf("use a constant value from file\n");
 	j_printf("     noise spectrum file = \"%s\"\n", ssload_filename);
       }
-      j_printf("\t     alpha coef. = %f\n", ssalpha);
-      j_printf("\t  spectral floor = %f\n", ssfloor);
+      j_printf("\t     alpha coef. = %f\n", para.ss_alpha);
+      j_printf("\t  spectral floor = %f\n", para.ss_floor);
     } else {
       j_printf("off\n");
     }
@@ -266,33 +270,45 @@ print_info()
    print_hmmdef_info(gmm);
   }
 
+  if (realtime_flag && para.cmn) {
+    j_printf("\nMAP-CMN on realtime input: \n");
+    if (cmnload_filename) {
+      if (cmn_loaded) {
+	j_printf("\t      initial CMN param = from \"%s\"\n", cmnload_filename);
+      } else {
+	j_printf("\t      initial CMN param = from \"%s\" (failed, ignored)\n", cmnload_filename);
+      }
+    } else {
+      j_printf("\t      initial CMN param = not specified\n");
+    }
+    j_printf("\t    initial mean weight = %6.2f\n", cmn_map_weight);
+    if (cmn_update) {
+      j_printf("\t       CMN param update = yes, update from last inputs\n");
+    } else {
+      j_printf("\t       CMN param update = no, keep initial\n");
+    }
+    if (cmnsave_filename) {
+      if (realtime_flag) {
+	j_printf("\t      save CMN param to = %s\n", cmnsave_filename);
+      } else {
+	j_printf("\t      save CMN param to = %s (not realtime CMN, ignored)\n", cmnsave_filename);
+      }
+    }
+  }
+
   j_printf("\nSearch parameters: \n");
     
   j_printf("\t      1st pass decoding = ");
   if (force_realtime_flag) j_printf("(forced) ");
   if (realtime_flag) {
     j_printf("on-the-fly");
-    if (cmn_required) j_printf(" with last CMN");
+    if (speech_input != SP_MFCFILE && para.cmn) j_printf(" with MAP-CMN");
     j_printf("\n");
   } else {
     j_printf("batch");
-    if (cmn_required) j_printf(" with sentence CMN");
+    if (speech_input != SP_MFCFILE && para.cmn) j_printf(" with sentence CMN");
     j_printf("\n");
   }
-  if (cmnload_filename) {
-    if (cmn_loaded) {
-      j_printf("\t initial CMN param from = %s\n", cmnload_filename);
-    } else {
-      j_printf("\t initial CMN param from = %s (failed, ignored)\n", cmnload_filename);
-    }
-  }
-  if (cmnsave_filename) {
-    if (realtime_flag) {
-      j_printf("\t      save CMN param to = %s\n", cmnsave_filename);
-    } else {
-      j_printf("\t      save CMN param to = %s (not realtime CMN, ignored)\n", cmnsave_filename);
-    }
-  }
   j_printf("\t        1st pass method = ");
 #ifdef WPAIR
 # ifdef WPAIR_KEEP_NLIMIT
@@ -357,10 +373,6 @@ print_info()
 
   j_printf("\t(-n)        search till = %d candidates found\n", nbest);
   j_printf("\t(-output)    and output = %d candidates out of above\n", output_hypo_maxnum);
-#ifdef GRAPHOUT
-  j_printf("\t(-graphrange)    margin = %d frames\n",graph_merge_neighbor_range);
-#endif
-  
   if (ccd_flag) {
     j_printf("\t IWCD handling:\n");
 #ifdef PASS1_IWCD
@@ -442,6 +454,37 @@ print_info()
 #endif
 #endif /* CONFIDENCE_MEASURE */
   
+#ifdef GRAPHOUT
+  j_printf("\nGraph output:\n");
+  j_printf("\t(-graphrange)    margin = %d frames", graph_merge_neighbor_range);
+  if (graph_merge_neighbor_range < 0) {
+    j_printf(" (all post-marging disabled)\n");
+  } else if (graph_merge_neighbor_range == 0) {
+    j_printf(" (merge same word with the same boundary)\n");
+  } else {
+    j_printf(" (merge same words around this margin)\n");
+  }
+#ifdef GRAPHOUT_DEPTHCUT
+  j_printf("\t(-graphcut)cutoff depth = ");
+  if (graphout_cut_depth < 0) {
+    j_printf("disabled (-1)\n");
+  } else {
+    j_printf("%d words\n",graphout_cut_depth);
+  }
+#endif
+#ifdef GRAPHOUT_LIMIT_BOUNDARY_LOOP
+  j_printf("\t(-graphboundloop)loopmax= %d for boundary adjustment\n",graphout_limit_boundary_loop_num);
+#endif
+#ifdef GRAPHOUT_SEARCH_DELAY_TERMINATION
+  j_printf("\tInhibit graph search termination before 1st sentence found = ");
+  if (graphout_search_delay) {
+    j_printf("enabled\n");
+  } else {
+    j_printf("disabled\n");
+  }
+#endif
+#endif /* GRAPHOUT */
+  
   j_printf("\nSystem I/O configuration:\n");
   j_printf("\t    speech input source = ");
   if (speech_input == SP_RAWFILE) {
@@ -481,9 +524,9 @@ print_info()
   }
   if (speech_input != SP_MFCFILE) {
     if (speech_input == SP_RAWFILE || speech_input == SP_STDIN || speech_input == SP_ADINNET) {
-      j_printf("\t         sampling freq. = %d Hz required\n", smpFreq);
+      j_printf("\t         sampling freq. = %d Hz required\n", para.smp_freq);
     } else {
-      j_printf("\t         sampling freq. = %d Hz\n", smpFreq);
+      j_printf("\t         sampling freq. = %d Hz\n", para.smp_freq);
     }
   }
   if (speech_input != SP_MFCFILE) {
@@ -513,13 +556,16 @@ print_info()
     } else {
       j_printf("\t        silence cutting = off\n");
     }
-    if (use_zmean) {
+    if (use_zmean || para.zmeanframe) {
       j_printf("\t       remove DC offset = on");
+      if (para.zmeanframe) {
+	j_printf(" (frame-wise)\n");
+      }
       if (speech_input == SP_RAWFILE) {
 	j_printf(" (will compute for each file)\n");
       } else {
 	j_printf(" (will compute from first %.1f sec)\n",
-		 (float)ZMEANSAMPLES / (float)smpFreq);
+		 (float)ZMEANSAMPLES / (float)para.smp_freq);
       }
     } else {
       j_printf("\t       remove DC offset = off\n");
@@ -569,17 +615,24 @@ print_info()
 
 #ifdef USE_MIC
   if (realtime_flag) {
-    if (cmn_required) {
+    if (para.cmn) {
       if (cmn_loaded) {
 	j_printf("\ninitial CMN parameter loaded from file\n");
       } else {
 	j_printf("\n");
-	j_printf("\t************************************************************\n");
-	j_printf("\t* NOTICE: The first input may not be correctly recoginized *\n");
-	j_printf("\t*         since no CMN parameter is available on startup.  *\n");
-	j_printf("\t************************************************************\n");
+	j_printf("\t*************************************************************\n");
+	j_printf("\t* NOTICE: The first input may not be recognized, since      *\n");
+	j_printf("\t*         no initial CMN parameter is available on startup. *\n");
+	j_printf("\t*************************************************************\n");
       }
     }
+    if (para.energy && para.enormal) {
+      j_printf("\t*************************************************************\n");
+      j_printf("\t* NOTICE: Energy normalization is activated on live input:  *\n");
+      j_printf("\t*         maximum energy of LAST INPUT will be used for it. *\n");
+      j_printf("\t*         So, the first input will not be recognized.       *\n");
+      j_printf("\t*************************************************************\n");
+    }
   }
 #endif
 }
