@@ -11,13 +11,13 @@
  * @brief  Read in a jconf file and process them.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -27,7 +27,7 @@
 #include <mbstring.h>
 #endif
 
-#define ISTOKEN(A) (A == ' ' || A == '\t' || A == '\n')
+#define ISTOKEN(A) (A == ' ' || A == '\t' || A == '\n') ///< Determine token characters
 
 /** 
  * <JA>
@@ -162,7 +162,8 @@ config_file_parse(char *conffile)
   char **c_argv;
   FILE *fp;
   int maxnum, step;
-  char buf[512], *cpy;
+  static const int len = 512;
+  char buf[len], cpy[len];
   char *p, *dst, *dst_from;
   char *cdir;
 
@@ -179,9 +180,8 @@ config_file_parse(char *conffile)
   c_argv = (char **)mymalloc(sizeof(char *) * maxnum);
   c_argv[0] = strcpy((char *)mymalloc(strlen(conffile)+1), conffile);
   c_argc = 1;
-  while (fgets_jconf(buf, 512, fp) != NULL) {
+  while (fgets_jconf(buf, len, fp) != NULL) {
     if (buf[0] == '\0') continue;
-    cpy = (char *)mymalloc(strlen(buf)+1);
     p = buf; dst = cpy;
     while (1) {
       while (*p != '\0' && ISTOKEN(*p)) p++;
@@ -190,21 +190,21 @@ config_file_parse(char *conffile)
       dst_from = dst;
       
       while (*p != '\0' && (!ISTOKEN(*p))) {
-	if (0 &&/* '\' is removed by fgets_jconf */ *p == 92) {     /* escape by '\' */
+	if (0 &&/* '\' is removed by fgets_jconf */ *p == '\\') {     /* escape by '\' */
 	  if (*(++p) == '\0') break;
 	  *(dst++) = *(p++);
 	} else {
-	  if (*p == 34) { /* quote by "" */
+	  if (*p == '"') { /* quote by "" */
 	    p++;
-	    while (*p != '\0' && *p != 34) *(dst++) = *(p++);
+	    while (*p != '\0' && *p != '"') *(dst++) = *(p++);
 	    if (*p == '\0') break;
 	    p++;
-	  } else if (*p == 39) { /* quote by '' */
+	  } else if (*p == '\'') { /* quote by '' */
 	    p++;
-	    while (*p != '\0' && *p != 39) *(dst++) = *(p++);
+	    while (*p != '\0' && *p != '\'') *(dst++) = *(p++);
 	    if (*p == '\0') break;
 	    p++;
-	  } else if (*p == 35) { /* comment out by '#' */
+	  } else if (*p == '#') { /* comment out by '#' */
 	    *p = '\0';
 	    break;
 	  } else {		/* other */
@@ -218,7 +218,7 @@ config_file_parse(char *conffile)
 	  maxnum += step;
 	  c_argv = (char **)myrealloc(c_argv, sizeof(char *) * maxnum);
 	}
-	c_argv[c_argc++] = dst_from;
+	c_argv[c_argc++] = strcpy((char*)mymalloc(strlen(dst_from)+1), dst_from);
       }
     }
   }
@@ -239,5 +239,11 @@ config_file_parse(char *conffile)
   get_dirname(cdir);
   opt_parse(c_argc, c_argv, (cdir[0] == '\0') ? NULL : cdir);
   free(cdir);
+
+  /* free arguments */
+  while (c_argc-- > 0) {
+    free(c_argv[c_argc]);
+  }
+  free(c_argv);
 }
 
