@@ -11,13 +11,13 @@
  * @brief  Process options and configurations from jconf or command argument.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.13 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -111,7 +111,7 @@ opt_parse(int argc, char *argv[], char *cwd)
   char *tmparg;
   int i;
 
-#define NEXTARG (++i >= argc) ? (char *)args_needed_exit(argv[i-1]) : argv[i]
+#define NEXTARG (++i >= argc) ? (char *)args_needed_exit(argv[i-1]) : argv[i] ///< Points to next argument, or error if not exist
 
   if (argc == 1) {		/* no argument */
     usage();
@@ -119,13 +119,21 @@ opt_parse(int argc, char *argv[], char *cwd)
   
   for (i=1;i<argc;i++) {
     if (strmatch(argv[i],"-C")) { /* include jconf file  */
-      config_file_parse(filepath(NEXTARG, cwd));
+      tmparg = filepath(NEXTARG, cwd);
+      config_file_parse(tmparg);
+      free(tmparg);
       continue;
     } else if (strmatch(argv[i],"-input")) { /* speech input */
       tmparg = NEXTARG;
-      if (strmatch(tmparg,"rawfile")) {
+      if (strmatch(tmparg,"file")) {
 	speech_input = SP_RAWFILE;
 	realtime_flag = FALSE;
+      } else if (strmatch(tmparg,"rawfile")) {
+	speech_input = SP_RAWFILE;
+	realtime_flag = FALSE;
+      } else if (strmatch(tmparg,"htkparam")) {
+	speech_input = SP_MFCFILE;
+	realtime_flag = FALSE;
       } else if (strmatch(tmparg,"mfcfile")) {
 	speech_input = SP_MFCFILE;
 	realtime_flag = FALSE;
@@ -157,10 +165,14 @@ opt_parse(int argc, char *argv[], char *cwd)
       }
       continue;
     } else if (strmatch(argv[i],"-filelist")) {	/* input file list */
-      inputlist_filename = NEXTARG;
+      if (inputlist_filename) free(inputlist_filename);
+      tmparg = NEXTARG;
+      inputlist_filename = strcpy((char*)mymalloc(strlen(tmparg)+1),tmparg);
       continue;
     } else if (strmatch(argv[i],"-record")) {	/* record speech data*/
-      record_dirname = NEXTARG;
+      if (record_dirname) free(record_dirname);
+      tmparg = NEXTARG;
+      record_dirname = strcpy((char*)mymalloc(strlen(tmparg)+1),tmparg);
       continue;
     } else if (strmatch(argv[i],"-rejectshort")) { /* short input rejection */
       rejectshortlen = atoi(NEXTARG);
@@ -249,7 +261,25 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-graphrange")) { /* neighbor merge range frame */
       graph_merge_neighbor_range = atoi(NEXTARG);
       continue;
+#ifdef GRAPHOUT_DEPTHCUT
+    } else if (strmatch(argv[i],"-graphcut")) { /* cut graph word by depth */
+      graphout_cut_depth = atoi(NEXTARG);
+      continue;
 #endif
+#ifdef GRAPHOUT_LIMIT_BOUNDARY_LOOP
+    } else if (strmatch(argv[i],"-graphboundloop")) { /* neighbor merge range frame */
+      graphout_limit_boundary_loop_num = atoi(NEXTARG);
+      continue;
+#endif
+#ifdef GRAPHOUT_SEARCH_DELAY_TERMINATION
+    } else if (strmatch(argv[i],"-graphsearchdelay")) { /* not do graph search termination before the 1st sentence is found */
+      graphout_search_delay = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-nographsearchdelay")) { /* not do graph search termination before the 1st sentence is found */
+      graphout_search_delay = FALSE;
+      continue;
+#endif
+#endif /* GRAPHOUT */
 #ifdef USE_DFA
     } else if (strmatch(argv[i],"-looktrellis")) { /* activate loopuprange */
       looktrellis_flag = TRUE;
@@ -306,37 +336,69 @@ opt_parse(int argc, char *argv[], char *cwd)
       tail_margin_msec = atoi(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-hipass")||strmatch(argv[i],"-hifreq")) { /* frequency of upper band limit */
-      hipass = atoi(NEXTARG);
+      para.hipass = atoi(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-lopass")||strmatch(argv[i],"-lofreq")) { /* frequency of lower band limit */
-      lopass = atoi(NEXTARG);
+      para.lopass = atoi(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-smpPeriod")) { /* sample period (ns) */
-      smpPeriod = atoi(NEXTARG);
-      smpFreq = period2freq(smpPeriod);
+      para.smp_period = atoi(NEXTARG);
+      para.smp_freq = period2freq(para.smp_period);
       continue;
     } else if (strmatch(argv[i],"-smpFreq")) { /* sample frequency (Hz) */
-      smpFreq = atoi(NEXTARG);
-      smpPeriod = freq2period(smpFreq);
+      para.smp_freq = atoi(NEXTARG);
+      para.smp_period = freq2period(para.smp_freq);
       continue;
     } else if (strmatch(argv[i],"-fsize")) { /* Window size */
-      fsize = atoi(NEXTARG);
+      para.framesize = atoi(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-fshift")) { /* Frame shiht */
-      fshift = atoi(NEXTARG);
+      para.frameshift = atoi(NEXTARG);
+      continue;
+    } else if (strmatch(argv[i],"-preemph")) {
+      para.preEmph = atof(NEXTARG);
+      continue;
+    } else if (strmatch(argv[i],"-fbank")) {
+      para.fbank_num = atoi(NEXTARG);
+      continue;
+    } else if (strmatch(argv[i],"-ceplif")) {
+      para.lifter = atoi(NEXTARG);
+      continue;
+    } else if (strmatch(argv[i],"-rawe")) {
+      para.raw_e = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-norawe")) {
+      para.raw_e = FALSE;
+      continue;
+    } else if (strmatch(argv[i],"-enormal")) {
+      para.enormal = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-noenormal")) {
+      para.enormal = FALSE;
+      continue;
+    } else if (strmatch(argv[i],"-escale")) {
+      para.escale = atof(NEXTARG);
+      continue;
+    } else if (strmatch(argv[i],"-silfloor")) {
+      para.silFloor = atof(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-delwin")) { /* Delta window length */
-      delwin = atoi(NEXTARG);
+      para.delWin = atoi(NEXTARG);
+      continue;
+    } else if (strmatch(argv[i],"-accwin")) { /* Acceleration window length */
+      para.accWin = atoi(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-ssalpha")) { /* alpha coef. for SS */
-      ssalpha = atof(NEXTARG);
+      para.ss_alpha = atof(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-ssfloor")) { /* spectral floor for SS */
-      ssfloor = atof(NEXTARG);
+      para.ss_floor = atof(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-version") || strmatch(argv[i], "--version") || strmatch(argv[i], "-setting") || strmatch(argv[i], "--setting")) { /* print version and exit */
       put_header(stderr);
       put_compile_defs(stderr);
+      j_printerr("\n");
+      put_library_defs(stderr);
       j_exit();
     } else if (strmatch(argv[i],"-quiet")) { /* minimum output */
       debug2_flag = verbose_flag = FALSE;
@@ -352,6 +414,8 @@ opt_parse(int argc, char *argv[], char *cwd)
       continue;
     } else if (strmatch(argv[i],"-nocharconv")) { /* disable character set conversion */
 #ifdef CHARACTER_CONVERSION
+      if (from_code) free(from_code);
+      if (to_code) free(to_code);
       from_code = NULL;
       to_code = NULL;
 #else
@@ -360,8 +424,12 @@ opt_parse(int argc, char *argv[], char *cwd)
       continue;
     } else if (strmatch(argv[i],"-charconv")) {	/* enable character set conversion */
 #ifdef CHARACTER_CONVERSION
-      from_code = NEXTARG;
-      to_code = NEXTARG;
+      if (from_code) free(from_code);
+      if (to_code) free(to_code);
+      tmparg = NEXTARG;
+      from_code = strcpy((char*)mymalloc(strlen(tmparg)+1),tmparg);
+      tmparg = NEXTARG;
+      to_code = strcpy((char*)mymalloc(strlen(tmparg)+1),tmparg);
 #else
       j_printerr("Warning: character set conversion disabled, option \"-charconv\" ignored\n");
       i+=2;
@@ -369,12 +437,14 @@ opt_parse(int argc, char *argv[], char *cwd)
       continue;
     } else if (strmatch(argv[i],"-kanji")) {
 #ifdef CHARACTER_CONVERSION
+      if (from_code) free(from_code);
+      if (to_code) free(to_code);
       from_code = NULL;
       tmparg = NEXTARG;
       if (strmatch(tmparg, "noconv")) {
 	to_code = NULL;
       } else {
-	to_code = tmparg;
+	to_code = strcpy((char*)mymalloc(strlen(tmparg)+1),tmparg);
       }
 #else
       j_printerr("Warning: character set conversion disabled, option \"-kanji\" ignored\n");
@@ -401,15 +471,20 @@ opt_parse(int argc, char *argv[], char *cwd)
       compute_only_1pass = TRUE;
       continue;
     } else if (strmatch(argv[i],"-hlist")) { /* HMM list file */
+      if (mapfilename) free(mapfilename);
       mapfilename = filepath(NEXTARG, cwd);
       continue;
 #ifdef USE_NGRAM
     } else if (strmatch(argv[i],"-nlr")) { /* word LR 2-gram (ARPA) */
+      if (ngram_filename_lr_arpa) free(ngram_filename_lr_arpa);
       ngram_filename_lr_arpa = filepath(NEXTARG, cwd);
+      if (ngram_filename) free(ngram_filename);
       ngram_filename = NULL;
       continue;
     } else if (strmatch(argv[i],"-nrl")) { /* word RL 3-gram (ARPA) */
+      if (ngram_filename_rl_arpa) free(ngram_filename_rl_arpa);
       ngram_filename_rl_arpa = filepath(NEXTARG, cwd);
+      if (ngram_filename) free(ngram_filename);
       ngram_filename = NULL;
       continue;
     } else if (strmatch(argv[i],"-lmp")) { /* LM weight and penalty (pass1) */
@@ -430,20 +505,23 @@ opt_parse(int argc, char *argv[], char *cwd)
       multigram_add_prefix_list(NEXTARG, cwd);
       continue;
     } else if (strmatch(argv[i],"-gramlist")) { /* file of grammar prefix list */
-      multigram_add_prefix_filelist(filepath(NEXTARG, cwd));
+      tmparg = filepath(NEXTARG, cwd);
+      multigram_add_prefix_filelist(tmparg);
+      free(tmparg);
       continue;
     } else if (strmatch(argv[i],"-nogram")) { /* remove grammar list */
       multigram_remove_gramlist();
-      if (dfa_filename != NULL) {
+      if (dfa_filename) {
 	free(dfa_filename);
 	dfa_filename = NULL;
       }
-      if (dictfilename != NULL) {
+      if (dictfilename) {
 	free(dictfilename);
 	dictfilename = NULL;
       }
       continue;
     } else if (strmatch(argv[i],"-dfa")) { /* DFA filename */
+      if (dfa_filename) free(dfa_filename);
       dfa_filename = filepath(NEXTARG, cwd);
       continue;
     } else if (strmatch(argv[i],"-penalty1")) {	/* word insertion penalty (pass1) */
@@ -454,7 +532,9 @@ opt_parse(int argc, char *argv[], char *cwd)
       continue;
 #endif
     } else if (strmatch(argv[i],"-spmodel") || strmatch(argv[i], "-sp")) { /* name of short pause word */
-      spmodel_name = NEXTARG;
+      if (spmodel_name) free(spmodel_name);
+      tmparg = NEXTARG;
+      spmodel_name = strcpy((char*)mymalloc(strlen(tmparg)+1),tmparg);
       continue;
 #ifdef MULTIPATH_VERSION
     } else if (strmatch(argv[i],"-iwsp")) { /* enable inter-word short pause handing */
@@ -466,16 +546,22 @@ opt_parse(int argc, char *argv[], char *cwd)
 #endif
 #ifdef USE_NGRAM
     } else if (strmatch(argv[i],"-silhead")) { /* head silence word name */
-      head_silname = NEXTARG;
+      if (head_silname) free(head_silname);
+      tmparg = NEXTARG;
+      head_silname = strcpy((char*)mymalloc(strlen(tmparg)+1),tmparg);
       continue;
     } else if (strmatch(argv[i],"-siltail")) { /* tail silence word name */
-      tail_silname = NEXTARG;
+      if (tail_silname) free(tail_silname);
+      tmparg = NEXTARG;
+      tail_silname = strcpy((char*)mymalloc(strlen(tmparg)+1),tmparg);
       continue;
     } else if (strmatch(argv[i],"-iwspword")) { /* add short pause word */
       enable_iwspword = TRUE;
       continue;
     } else if (strmatch(argv[i],"-iwspentry")) { /* content of the iwspword */
-      iwspentry = NEXTARG;
+      if (iwspentry) free(iwspentry);
+      tmparg = NEXTARG;
+      iwspentry = strcpy((char*)mymalloc(strlen(tmparg)+1),tmparg);
       continue;
     } else if (strmatch(argv[i],"-iwcache")) { /* control cross-word LM cache */
 #ifdef HASH_CACHE_IW
@@ -497,7 +583,9 @@ opt_parse(int argc, char *argv[], char *cwd)
 #endif /* USE_NGRAM */
 #ifdef USE_NETAUDIO
     } else if (strmatch(argv[i],"-NA")) { /* netautio device name */
-      netaudio_devname = NEXTARG;
+      if (netaudio_devname) free(netaudio_devname);
+      tmparg = NEXTARG;
+      netaudio_devname = strcpy((char*)mymalloc(strlen(tmparg)+1),tmparg);
       continue;
 #endif
     } else if (strmatch(argv[i],"-adport")) { /* adinnet port num */
@@ -512,6 +600,12 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-nozmean")) { /* disable DC offset by zero mean */
       use_zmean = FALSE;
       continue;
+    } else if (strmatch(argv[i],"-zmeanframe")) { /* enable frame-wise DC offset by zero mean */
+      para.zmeanframe = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-nozmeanframe")) { /* disable frame-wise DC offset by zero mean */
+      para.zmeanframe = FALSE;
+      continue;
 #ifdef SP_BREAK_CURRENT_FRAME
     } else if (strmatch(argv[i],"-spdur")) { /* short-pause duration threshold */
       sp_frame_duration = atoi(NEXTARG);
@@ -571,31 +665,46 @@ opt_parse(int argc, char *argv[], char *cwd)
       enveloped_bestfirst_width = atoi(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-hgs")) { /* Gaussian selection model file */
+      if (hmm_gs_filename) free(hmm_gs_filename);
       hmm_gs_filename = filepath(NEXTARG, cwd);
       continue;
     } else if (strmatch(argv[i],"-booknum")) { /* num of state to select in GS */
       gs_statenum = atoi(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-gshmm")) { /* same as "-hgs" */
+      if (hmm_gs_filename) free(hmm_gs_filename);
       hmm_gs_filename = filepath(NEXTARG, cwd);
       continue;
     } else if (strmatch(argv[i],"-gsnum")) { /* same as "-booknum" */
       gs_statenum = atoi(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-cmnload")) { /* load CMN parameter from file */
+      if (cmnload_filename) free(cmnload_filename);
       cmnload_filename = filepath(NEXTARG, cwd);
       continue;
     } else if (strmatch(argv[i],"-cmnsave")) { /* save CMN parameter to file */
+      if (cmnsave_filename) free(cmnsave_filename);
       cmnsave_filename = filepath(NEXTARG, cwd);
       continue;
+    } else if (strmatch(argv[i],"-cmnupdate")) { /* update CMN parameter */
+      cmn_update = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-cmnnoupdate")) { /* not update CMN parameter */
+      cmn_update = FALSE;
+      continue;
+    } else if (strmatch(argv[i],"-cmnmapweight")) { /* CMN weight for MAP */
+      cmn_map_weight = (float)atof(NEXTARG);
+      continue;
     } else if (strmatch(argv[i],"-sscalc")) { /* do spectral subtraction (SS) for raw file input */
       sscalc = TRUE;
+      if (ssload_filename) free(ssload_filename);
       ssload_filename = NULL;
       continue;
     } else if (strmatch(argv[i],"-sscalclen")) { /* head silence length used to compute SS (in msec) */
       sscalc_len = atoi(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-ssload")) { /* load SS parameter from file */
+      if (ssload_filename) free(ssload_filename);
       ssload_filename = filepath(NEXTARG, cwd);
       sscalc = FALSE;
       continue;
@@ -623,6 +732,7 @@ opt_parse(int argc, char *argv[], char *cwd)
 #endif
 #endif /* CONFIDENCE_MEASURE */
     } else if (strmatch(argv[i],"-gmm")) { /* load SS parameter from file */
+      if (gmm_filename) free(gmm_filename);
       gmm_filename = filepath(NEXTARG, cwd);
       continue;
     } else if (strmatch(argv[i],"-gmmnum")) { /* num of Gaussian pruning for GMM */
@@ -630,8 +740,13 @@ opt_parse(int argc, char *argv[], char *cwd)
       continue;
     } else if (strmatch(argv[i],"-gmmreject")) {
       tmparg = NEXTARG;
-      gmm_reject_cmn_string = (char *)mymalloc(strlen(tmparg)+1);
-      strcpy(gmm_reject_cmn_string, tmparg);
+      gmm_reject_cmn_string = strcpy((char *)mymalloc(strlen(tmparg)+1), tmparg);
+      continue;
+    } else if (strmatch(argv[i],"-htkconf")) {
+      tmparg = NEXTARG;
+      if (htk_config_file_parse(tmparg, &para_htk) == FALSE) {
+	j_error("Error: failed to read %s\n", tmparg);
+      }
       continue;
     } else if (strmatch(argv[i],"-help")) { /* output version and option */
       detailed_help();
@@ -642,13 +757,18 @@ opt_parse(int argc, char *argv[], char *cwd)
       /* 1-letter options */
       switch(argv[i][1]) {
       case 'h':			/* hmmdefs */
+	if (hmmfilename) free(hmmfilename);
 	hmmfilename = filepath(NEXTARG, cwd);
 	break;
       case 'v':			/* dictionary */
+	if (dictfilename) free(dictfilename);
 	dictfilename = filepath(NEXTARG, cwd);
 	break;
 #ifdef USE_NGRAM
       case 'd':			/* binary N-gram */
+	if (ngram_filename) free(ngram_filename);
+	if (ngram_filename_lr_arpa) free(ngram_filename_lr_arpa);
+	if (ngram_filename_rl_arpa) free(ngram_filename_rl_arpa);
 	ngram_filename = filepath(NEXTARG, cwd);
 	ngram_filename_lr_arpa = NULL;
 	ngram_filename_rl_arpa = NULL;
@@ -675,5 +795,141 @@ opt_parse(int argc, char *argv[], char *cwd)
       opt_terminate();
     }
   }
+
+  /* set default values if not specified yet */
+  if (!spmodel_name) {
+    spmodel_name = strcpy((char*)mymalloc(strlen(SPMODEL_NAME_DEFAULT)+1),
+			  SPMODEL_NAME_DEFAULT);
+  }
+#ifdef USE_NGRAM
+  if (!head_silname) {
+    head_silname = strcpy((char*)mymalloc(strlen(BEGIN_WORD_DEFAULT)+1),
+			  BEGIN_WORD_DEFAULT);
+  }
+  if (!tail_silname) {
+    tail_silname = strcpy((char*)mymalloc(strlen(END_WORD_DEFAULT)+1),
+			  END_WORD_DEFAULT);
+  }
+  if (!iwspentry) {
+    iwspentry = strcpy((char*)mymalloc(strlen(IWSPENTRY_DEFAULT)+1),
+		       IWSPENTRY_DEFAULT);
+  }
+#endif	/* USE_NGRAM */
+#ifdef USE_NETAUDIO
+  if (!netaudio_devname) {
+    netaudio_devname = strcpy((char*)mymalloc(strlen(NETAUDIO_DEVNAME)+1),
+			      NETAUDIO_DEVNAME);
+  }
+#endif	/* USE_NETAUDIO */
 }
 
+/** 
+ * <JA>
+ * ץϢΥХѿΥΰ롥
+ * </JA>
+ * <EN>
+ * Free memories of global variables allocated by option arguments.
+ * </EN>
+ */
+void
+opt_release()
+{
+  if (inputlist_filename) {
+    free(inputlist_filename);
+    inputlist_filename = NULL;
+  }
+  if (record_dirname) {
+    free(record_dirname);
+    record_dirname = NULL;
+  }
+#ifdef CHARACTER_CONVERSION
+  if (from_code) {
+    free(from_code);
+    from_code = NULL;
+  }
+  if (to_code) {
+    free(to_code);
+    to_code = NULL;
+  }
+#endif	/* CHARACTER_CONVERSION */
+  if (mapfilename) {
+    free(mapfilename);
+    mapfilename = NULL;
+  }
+#ifdef USE_NGRAM
+  if (ngram_filename) {
+    free(ngram_filename);
+    ngram_filename = NULL;
+  }
+  if (ngram_filename_lr_arpa) {
+    free(ngram_filename_lr_arpa);
+    ngram_filename_lr_arpa = NULL;
+  }
+  if (ngram_filename_rl_arpa) {
+    free(ngram_filename_rl_arpa);
+    ngram_filename_rl_arpa = NULL;
+  }
+#endif	/* USE_NGRAM */
+#ifdef USE_DFA
+  if (dfa_filename) {
+    free(dfa_filename);
+    dfa_filename = NULL;
+  }
+#endif	/* USE_DFA */
+  if (spmodel_name) {
+    free(spmodel_name);
+    spmodel_name = NULL;
+  }
+#ifdef USE_NGRAM
+  if (head_silname) {
+    free(head_silname);
+    head_silname = NULL;
+  }
+  if (tail_silname) {
+    free(tail_silname);
+    tail_silname = NULL;
+  }
+  if (iwspentry) {
+    free(iwspentry);
+    iwspentry = NULL;
+  }
+#endif	/* USE_NGRAM */
+#ifdef USE_NETAUDIO
+  if (netaudio_devname) {
+    free(netaudio_devname);
+    netaudio_devname = NULL;
+  }
+#endif	/* USE_NETAUDIO */
+  if (hmm_gs_filename) {
+    free(hmm_gs_filename);
+    hmm_gs_filename = NULL;
+  }
+  if (cmnload_filename) {
+    free(cmnload_filename);
+    cmnload_filename = NULL;
+  }
+  if (cmnsave_filename) {
+    free(cmnsave_filename);
+    cmnsave_filename = NULL;
+  }
+  if (ssload_filename) {
+    free(ssload_filename);
+    ssload_filename = NULL;
+  }
+  if (gmm_filename) {
+    free(gmm_filename);
+    gmm_filename = NULL;
+  }
+  if (gmm_reject_cmn_string) {
+    free(gmm_reject_cmn_string);
+    gmm_reject_cmn_string = NULL;
+  }
+  if (hmmfilename) {
+    free(hmmfilename);
+    hmmfilename = NULL;
+  }
+  if (dictfilename) {
+    free(dictfilename);
+    dictfilename = NULL;
+  }
+}
