@@ -20,12 +20,12 @@
  * It needs gtk version = 1.x, (tested on gtk-1.2).
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
  * Copyright (c) 2003-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -1589,7 +1589,7 @@ action_play_waveform(GtkWidget *widget)
   }
   close(fd);
   
-  snprintf(command, 250, PLAYCOMMAND, smpFreq, buf);
+  snprintf(command, 250, PLAYCOMMAND, para.smp_freq, buf);
   j_printf("play: [%s]\n", command);
   system(command);
 
