@@ -4,11 +4,11 @@
  * @date   Sun Sep 18 19:40:34 2005
  * 
  * <JA>
- * @brief  ȷħ٥ȥѴ롥
+ * @brief  ȷħ٥ȥ(MFCC)Ѵ롥
  *
  * Ϥ줿ȷ顤ħ٥ȥФޤ
- * Julius/JulianФǤħ٥ȥϡ
- * MFCC_{0|E}_D[_Z][_N]  {25|26} ΤΤΤߤǤ
+ * Julius/JulianФǤħ٥ȥϡMFCC ǤռΤΤǡ
+ * _0, _E, _D, _A, _Z, _N Ǥդȹ碌򥵥ݡȤޤ
  * ΤۤĹե졼ॷեȡӰ襫åȤʤɤΥѥ᡼Ǥޤ
  * ǧˤϡǥΥإåȥåԤ졤CMN̵ͭʤ
  * ꤵޤ
@@ -20,12 +20,12 @@
  * </JA>
  * 
  * <EN>
- * @brief  Convert speech waveform to parameter vector sequence.
+ * @brief  Convert speech waveform to MFCC parameter vector sequence.
  *
  * Parameter vector sequence extraction of input speech is done
- * here.  Only parameter format of MFCC_{0|E}_D[_Z][_N] with {25|26}
- * dimensions can be extracted within Julius/Julian, so acoustic model
- * for recognition should be trained by one of these parameter types.
+ * here.  The supported parameter is MFCC, with any combination of
+ * all the qualifiers in HTK: _0, _E, _D, _A, _Z, _N.  Acoustic model
+ * for recognition should be trained with the same parameter type.
  * You can specify other parameters such as window size, frame shift,
  * high/low frequency cut-off via runtime options.  At startup, Julius
  * will check for the parameter types of acoustic model if it conforms
@@ -39,13 +39,13 @@
  * within functions in realtime-1stpass.c instead of these.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.7 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -55,11 +55,10 @@
 
 /* make MFCC_E_D_N_Z from speech[0..speechlen-1] and returns
    newly malloced param */
+
 /** 
  * <JA>
- * ȷǡ 25 MFCC_E_D_N_Z (MFCC12++power +
- * ХåեΤǤ CMN)
- * ħѥ᡼Ф롥
+ * ȷǡ MFCC ѥ᡼Ф롥
  * 
  * @param speech [in] ȷǡ
  * @param speechlen [in] @a speech Ĺñ̡ץ
@@ -68,8 +67,7 @@
  * ѥ᡼¤ΤؤΥݥ󥿤֤
  * </JA>
  * <EN>
- * Extract MFCC_E_D_N_Z parameters (MFCC(12 dim.) + delta + delta power)
- * with sentence CMN from given waveform.
+ * Extract MFCC parameters with sentence CMN from given waveform.
  * 
  * @param speech [in] buffer of speech waveform
  * @param speechlen [in] length of @a speech in samples
@@ -81,36 +79,10 @@
 HTK_Param *new_wav2mfcc(SP16 speech[], int speechlen)
 {
   HTK_Param *param;
-  Value para;			/* parameters for Wav2MFCC */
   int framenum;
   int i;
   int len;
 
-  /* set parameters */
-  para.smp_period = smpPeriod;
-  para.framesize  = fsize;
-  para.frameshift = fshift;
-  para.preEmph    = DEF_PREENPH;
-  para.mfcc_dim   = DEF_MFCCDIM;
-  para.lifter     = DEF_CEPLIF;
-  para.delWin     = delwin;
-  para.silFloor   = DEF_SILFLOOR;
-  para.hipass     = hipass;
-  para.lopass     = lopass;
-  para.c0         = c0_required;
-  /* para.escale     = DEF_ESCALE; */
-  para.escale     = 1.0;
-  para.fbank_num  = DEF_FBANK;
-  /* para.cmn        = FALSE;*/
-  para.cmn        = cmn_required;
-  para.enormal    = FALSE;
-  para.raw_e      = FALSE;
-  /* for SS */
-  para.ss_alpha   = ssalpha;
-  para.ss_floor   = ssfloor;
-
-  para.vec_num = (para.mfcc_dim + 1) * 2;
-
   if (ssload_filename && ssbuf == NULL) {
     /* load noise spectrum for spectral subtraction from file (once) */
     if ((ssbuf = new_SS_load_from_file(ssload_filename, &sslen)) == NULL) {
@@ -120,7 +92,7 @@ HTK_Param *new_wav2mfcc(SP16 speech[], int speechlen)
 
   if (sscalc) {
     /* compute noise spectrum from head silence for each input */
-    len = sscalc_len * smpFreq / 1000;
+    len = sscalc_len * para.smp_freq / 1000;
     if (len > speechlen) len = speechlen;
 #ifdef SSDEBUG
     printf("[%d]\n", len);
@@ -147,27 +119,24 @@ HTK_Param *new_wav2mfcc(SP16 speech[], int speechlen)
   param = new_param();
   param->parvec = (VECT **)mymalloc(sizeof(VECT *) * framenum);
   for(i=0;i<framenum;i++) {
-    param->parvec[i] = (VECT *)mymalloc(sizeof(VECT) * para.vec_num);
+    param->parvec[i] = (VECT *)mymalloc(sizeof(VECT) * para.veclen);
   }
 
-  /* make MFCC_E_D_(N_)_Z from speech data */
-  /* (bogus) needs conversion here if intergerized */
-  Wav2MFCC_E_D(speech, param->parvec, para, speechlen, ssbuf, sslen);
+  /* make MFCC from speech data */
+  Wav2MFCC(speech, param->parvec, para, speechlen, ssbuf, sslen);
 
   /* set miscellaneous parameters */
   param->header.samplenum = framenum;
   param->header.wshift = para.smp_period * para.frameshift;
-  param->header.sampsize = para.vec_num * sizeof(VECT); /* not compressed */
-  param->header.samptype = F_MFCC | F_DELTA;
-  if (para.c0) {
-    param->header.samptype |= F_ZEROTH;
-  } else {
-    param->header.samptype |= F_ENERGY;
-  }
-  if (para.cmn) {
-    param->header.samptype |= F_CEPNORM;
-  }
-  param->veclen = para.vec_num;
+  param->header.sampsize = para.veclen * sizeof(VECT); /* not compressed */
+  param->header.samptype = F_MFCC;
+  if (para.delta) param->header.samptype |= F_DELTA;
+  if (para.acc) param->header.samptype |= F_ACCL;
+  if (para.energy) param->header.samptype |= F_ENERGY;
+  if (para.c0) param->header.samptype |= F_ZEROTH;
+  if (para.absesup) param->header.samptype |= F_ENERGY_SUP;
+  if (para.cmn) param->header.samptype |= F_CEPNORM;
+  param->veclen = para.veclen;
   param->samplenum = framenum;
 
   return param;
