@@ -30,13 +30,13 @@
  * "tree lexicon". 
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -995,13 +995,13 @@ wchmm_add_word(WCHMM_INFO *wchmm, int word, int matchlen, int matchword)
 		   can follow this word grammatically.
 		   so fallback to normal state */
 		j_printerr("Warning: no lcdset found for [%s::%d], fallback to [%s]\n", ltmp->name, wchmm->winfo->wton[word], ltmp->name);
-		lcd = lcdset_lookup_by_hmmname(hmminfo, ltmp->name);
+		lcd = lcdset_lookup_by_hmmname(wchmm->hmminfo, ltmp->name);
 	      }
 	    } else {
-	      lcd = lcdset_lookup_by_hmmname(hmminfo, ltmp->name);
+	      lcd = lcdset_lookup_by_hmmname(wchmm->hmminfo, ltmp->name);
 	    }
 #else
-	    lcd = lcdset_lookup_by_hmmname(hmminfo, ltmp->name);
+	    lcd = lcdset_lookup_by_hmmname(wchmm->hmminfo, ltmp->name);
 #endif
 	    if (lcd == NULL) {
 	      j_error("Error: no lcdset found for [%s]\n",ltmp->name);
@@ -1151,19 +1151,19 @@ wchmm_add_word(WCHMM_INFO *wchmm, int word, int matchlen, int matchword)
     
     /* set short pause state info */
     ntmp_bak = ntmp;
-    if (hmminfo->sp->is_pseudo) {
-      for(k = 1;k < hmm_logical_state_num(hmminfo->sp) - 1; k++) {
+    if (wchmm->hmminfo->sp->is_pseudo) {
+      for(k = 1;k < hmm_logical_state_num(wchmm->hmminfo->sp) - 1; k++) {
 	wchmm->outstyle[ntmp] = AS_LSET;
-	wchmm->state[ntmp].out.lset = &(hmminfo->sp->body.pseudo->stateset[k]);
+	wchmm->state[ntmp].out.lset = &(wchmm->hmminfo->sp->body.pseudo->stateset[k]);
 	wchmm->state[ntmp].ac = NULL;
 	wchmm->stend[ntmp] = WORD_INVALID;
 	ntmp++;
 	if (ntmp >= wchmm->maxwcn) wchmm_extend(wchmm);
       }
     } else {
-      for(k = 1;k < hmm_logical_state_num(hmminfo->sp) - 1; k++) {
+      for(k = 1;k < hmm_logical_state_num(wchmm->hmminfo->sp) - 1; k++) {
 	wchmm->outstyle[ntmp] = AS_STATE;
-	wchmm->state[ntmp].out.state = hmminfo->sp->body.defined->s[k];
+	wchmm->state[ntmp].out.state = wchmm->hmminfo->sp->body.defined->s[k];
 	wchmm->state[ntmp].ac = NULL;
 	wchmm->stend[ntmp] = WORD_INVALID;
 	ntmp++;
@@ -1173,13 +1173,13 @@ wchmm_add_word(WCHMM_INFO *wchmm, int word, int matchlen, int matchword)
     ntmp = ntmp_bak;
     /* connect incoming arcs from previous phone */
     out_num_next = 0;
-    for (ato = 1; ato < hmm_logical_state_num(hmminfo->sp); ato++) {
-      prob = hmm_logical_trans(hmminfo->sp)->a[0][ato];
+    for (ato = 1; ato < hmm_logical_state_num(wchmm->hmminfo->sp); ato++) {
+      prob = hmm_logical_trans(wchmm->hmminfo->sp)->a[0][ato];
       if (prob != LOG_ZERO) {
 	/* to control short pause insertion, transition probability toward
 	 the word-end short pause will be given a penalty */
-	prob += hmminfo->iwsp_penalty;
-	if (ato == hmm_logical_state_num(hmminfo->sp) - 1) {
+	prob += wchmm->hmminfo->iwsp_penalty;
+	if (ato == hmm_logical_state_num(wchmm->hmminfo->sp) - 1) {
 	  /* model has a model skip transition, just inherit them to next */
 	  for(kkk=0; kkk<out_num_prev; kkk++) {
 	    out_from_next[out_num_next] = out_from[kkk];
@@ -1195,7 +1195,7 @@ wchmm_add_word(WCHMM_INFO *wchmm, int word, int matchlen, int matchword)
       }
     }
     /* if short pause model doesn't have a model skip transition, also add it */
-    if (hmm_logical_trans(hmminfo->sp)->a[0][hmm_logical_state_num(hmminfo->sp)-1] == LOG_ZERO) {
+    if (hmm_logical_trans(wchmm->hmminfo->sp)->a[0][hmm_logical_state_num(wchmm->hmminfo->sp)-1] == LOG_ZERO) {
       /* to make insertion sp model to have no effect on the original path,
 	 the skip transition probability should be 0.0 (=100%) */
       prob = 0.0;
@@ -1206,11 +1206,11 @@ wchmm_add_word(WCHMM_INFO *wchmm, int word, int matchlen, int matchword)
       }
     }
     /* connect arcs within model, and store new outgoing arcs for wordend node */
-    for (k = 1; k < hmm_logical_state_num(hmminfo->sp) - 1; k++) {
-      for (ato = 1; ato < hmm_logical_state_num(hmminfo->sp); ato++) {
-	prob = hmm_logical_trans(hmminfo->sp)->a[k][ato];
+    for (k = 1; k < hmm_logical_state_num(wchmm->hmminfo->sp) - 1; k++) {
+      for (ato = 1; ato < hmm_logical_state_num(wchmm->hmminfo->sp); ato++) {
+	prob = hmm_logical_trans(wchmm->hmminfo->sp)->a[k][ato];
 	if (prob != LOG_ZERO) {
-	  if (ato == hmm_logical_state_num(hmminfo->sp) - 1) {
+	  if (ato == hmm_logical_state_num(wchmm->hmminfo->sp) - 1) {
 	    out_from_next[out_num_next] = ntmp;
 	    out_a_next[out_num_next] = prob;
 	    out_num_next++;
@@ -1246,7 +1246,7 @@ wchmm_add_word(WCHMM_INFO *wchmm, int word, int matchlen, int matchword)
 
   /* paranoia check if the short-pause addition has been done well */
   if (enable_iwsp && add_tail - add_head + 1 > 0) {
-    n += hmm_logical_state_num(hmminfo->sp) - 2;
+    n += hmm_logical_state_num(wchmm->hmminfo->sp) - 2;
     if (n != ntmp) j_error("Algorithm Error! cannot match\n");
   }
   
@@ -1520,7 +1520,7 @@ build_wchmm(WCHMM_INFO *wchmm)
 #ifdef CATEGORY_TREE
   if (ccd_flag && !old_iwcd_flag) {
     /* ƤΥƥIDդ lcd_set  */
-    lcdset_register_with_category_all(wchmm, hmminfo, winfo, dfa);
+    lcdset_register_with_category_all(wchmm, wchmm->hmminfo, winfo, dfa);
   }
 #endif /* CATEGORY_TREE */
 #endif /* PASS1_IWCD */
@@ -1723,7 +1723,7 @@ build_wchmm2(WCHMM_INFO *wchmm)
     /* when Julian mode (category-tree) and triphone is used,
        make all category-indexed context-dependent phone set (cdset) here */
     /* these will be assigned on the last phone of each word on tree */
-    lcdset_register_with_category_all(wchmm, hmminfo, winfo, dfa);
+    lcdset_register_with_category_all(wchmm, wchmm->hmminfo, winfo, dfa);
   }
 #endif /* CATEGORY_TREE */
 #endif /* PASS1_IWCD */
