@@ -20,13 +20,13 @@
  *
  * </EN>
  *
- * $Revision:$
+ * $Revision: 1.7 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -75,6 +75,7 @@ size_t myfread(void *ptr, size_t size, size_t n, FILE *fp);
 size_t myfwrite(void *ptr, size_t size, size_t n, FILE *fp);
 int myfgetc(FILE *fp);
 int myfeof(FILE *fp);
+int myfrewind(FILE *fp);
 
 /* mybmalloc.c */
 void mybmalloc_set_param();
@@ -82,6 +83,7 @@ void *mybmalloc(int);
 void mybmalloc_free(void);
 char *mybstrdup(char *);
 void *mybmalloc2(int size, BMALLOC_BASE **list);
+char *mybstrdup2(char *, BMALLOC_BASE **list);
 void mybfree2(BMALLOC_BASE **list);
 
 /* mymalloc.c */
@@ -127,4 +129,13 @@ char *mystrtok_quote(char *str, char *delim);
 char *mystrtok(char *str, char *delim);
 char *mystrtok_movetonext(char *str, char *delim);
 
+/* confout.c */
+void confout(FILE *strm);
+void confout_version(FILE *strm);
+void confout_audio(FILE *strm);
+void confout_lm(FILE *strm);
+void confout_am(FILE *strm);
+void confout_lib(FILE *strm);
+void confout_process(FILE *strm);
+
 #endif /* __SENT_UTIL_H__ */
