@@ -33,13 +33,13 @@
  * to the logical %HMM mapping function is also done here.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -134,8 +134,8 @@ hmm_add_physical_to_logical(HTK_HMM_INFO *hmminfo)
     }
     /* create new HMM_Logical */
     /* body refers to the physical HMM */
-    new = (HMM_Logical *)mybmalloc(sizeof(HMM_Logical));
-    new->name = (char *)mybmalloc(strlen(ph->name) + 1);
+    new = (HMM_Logical *)mybmalloc2(sizeof(HMM_Logical), &(hmminfo->lroot));
+    new->name = (char *)mybmalloc2(strlen(ph->name) + 1, &(hmminfo->lroot));
     strcpy(new->name, ph->name);
     new->is_pseudo = FALSE;
     new->body.defined = ph;
@@ -180,8 +180,8 @@ hmm_add_pseudo_phones_sub(HTK_HMM_INFO *hmminfo, char *name)
       }*/
   } else {
     /* create new HMM_Logical with pseudo body */
-    new = (HMM_Logical *)mybmalloc(sizeof(HMM_Logical));
-    new->name = (char *)mybmalloc(strlen(name) + 1);
+    new = (HMM_Logical *)mybmalloc2(sizeof(HMM_Logical), &(hmminfo->lroot));
+    new->name = (char *)mybmalloc2(strlen(name) + 1, &(hmminfo->lroot));
     strcpy(new->name, name);
     new->is_pseudo = TRUE;
     new->body.pseudo = cdset_lookup(hmminfo, name);
