@@ -29,13 +29,13 @@
  * variable to read the current token.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.5 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -118,47 +118,26 @@ conv_log_arc(HTK_HMM_INFO *hmm)
     }
   }
 }
-
-
 /** 
- * Initialize the %HMM definition data by setting default values.
+ * Invert all the variance values.
  * 
- * @param hmm [i/o] %HMM definition data to initialize.
+ * @param hmm [i/o] %HMM definition data to modify.
  */
 void
-init_hmm(HTK_HMM_INFO *hmm)
+htk_hmm_inverse_variances(HTK_HMM_INFO *hmm)
 {
-  /* set default options */
-  hmm->opt.stream_info.num = 1;
-  hmm->opt.cov_type = C_DIAG_C;
-  hmm->opt.dur_type = D_NULL;
+  HTK_HMM_Var *v;
+  int i,j;
+  LOGPROB l;
 
-  hmm->trstart = NULL;
-  hmm->vrstart = NULL;
-  hmm->ststart = NULL;
-  hmm->dnstart = NULL;
-  hmm->start   = NULL;
-  hmm->lgstart = NULL;
-  hmm->physical_root = NULL;
-  hmm->logical_root = NULL;
-  hmm->tr_root = NULL;
-  hmm->vr_root = NULL;
-  hmm->dn_root = NULL;
-  hmm->st_root = NULL;
-  hmm->codebooknum = 0;
-  hmm->codebook_root = NULL;
-  hmm->maxcodebooksize = 0;
-  hmm->totalmixnum = 0;
-  hmm->totalstatenum = 0;
-  hmm->totalhmmnum = 0;
-  hmm->totallogicalnum = 0;
-  hmm->is_triphone = FALSE;
-  hmm->is_tied_mixture = FALSE;
-  hmm->cdset_method = IWCD_NBEST;
-  hmm->cdmax_num = 3;
-  hmm->totalpseudonum = 0;
+  for (v = hmm->vrstart; v; v = v->next) {
+    for(i=0;i<v->len;i++) {
+      v->vec[i] = 1 / v->vec[i];
+    }
+  }
 }
 
+
 /** 
  * @brief  Main top routine to read in HTK %HMM definition file.
  *
@@ -176,9 +155,9 @@ rdhmmdef(FILE *fp, HTK_HMM_INFO *hmm)
   char macrosw;
   char *name;
 
-  /* initialize hmm data */
-  init_hmm(hmm);
-  
+  /* variances in htkdefs are not inversed yet */
+  hmm->variance_inversed = FALSE;
+
   /* read the first token */
   read_token(fp);
   
@@ -194,37 +173,37 @@ rdhmmdef(FILE *fp, HTK_HMM_INFO *hmm)
       set_global_opt(fp,hmm);
       break;
     case 't':			/* transition macro */
-      name = mybstrdup(rdhmmdef_token);
+      name = mybstrdup2(rdhmmdef_token, &(hmm->mroot));
       if (strlen(name) >= MAX_HMMNAME_LEN) rderr("Macro name too long");
       read_token(fp);
       def_trans_macro(name, fp, hmm);
       break;
     case 's':			/* state macro */
-      name = mybstrdup(rdhmmdef_token);
+      name = mybstrdup2(rdhmmdef_token, &(hmm->mroot));
       if (strlen(name) >= MAX_HMMNAME_LEN) rderr("Macro name too long");
       read_token(fp);
       def_state_macro(name, fp, hmm);
       break;
     case 'm':			/* density (mixture) macro */
-      name = mybstrdup(rdhmmdef_token);
+      name = mybstrdup2(rdhmmdef_token, &(hmm->mroot));
       if (strlen(name) >= MAX_HMMNAME_LEN) rderr("Macro name too long");
       read_token(fp);
       def_dens_macro(name, fp, hmm);
       break;
     case 'h':			/* HMM define */
-      name = mybstrdup(rdhmmdef_token);
+      name = mybstrdup2(rdhmmdef_token, &(hmm->mroot));
       if (strlen(name) >= MAX_HMMNAME_LEN) rderr("Macro name too long");
       read_token(fp);
       def_HMM(name, fp, hmm);
       break;
     case 'v':			/* Variance macro */
-      name = mybstrdup(rdhmmdef_token);
+      name = mybstrdup2(rdhmmdef_token, &(hmm->mroot));
       if (strlen(name) >= MAX_HMMNAME_LEN) rderr("Macro name too long");
       read_token(fp);
       def_var_macro(name, fp, hmm);
       break;
     case 'r':			/* Regression class macro (ignore) */
-      name = mybstrdup(rdhmmdef_token);
+      name = mybstrdup2(rdhmmdef_token, &(hmm->mroot));
       if (strlen(name) >= MAX_HMMNAME_LEN) rderr("Macro name too long");
       read_token(fp);
       def_regtree_macro(name, fp, hmm);
@@ -242,6 +221,12 @@ rdhmmdef(FILE *fp, HTK_HMM_INFO *hmm)
   }
   /* convert transition prob to log scale */
   conv_log_arc(hmm);
+
+  /* inverse all variance values for faster computation */
+  if (! hmm->variance_inversed) {
+    htk_hmm_inverse_variances(hmm);
+    hmm->variance_inversed = TRUE;
+  }
   
   /* check HMM parameter option type */
   if (!check_hmm_options(hmm)) {
