@@ -11,13 +11,13 @@
  * @brief Compute weighed sum of Gaussian mixture for non tied-mixture model (no cache)
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -53,5 +53,5 @@ calc_mix()
   }
   logprob = addlog_array(OP_calced_score, OP_calced_num);
   if (logprob <= LOG_ZERO) return LOG_ZERO;
-  return (logprob / LOG_TEN);
+  return (logprob * INV_LOG_TEN);
 }
