@@ -11,13 +11,13 @@
  * @brief  Calculate the GMS monophone %HMM for Gaussian Mixture Selection using Gaussian pruning
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.6 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -60,10 +60,10 @@ void
 gms_gprune_init(HTK_HMM_INFO *hmminfo, int gsset_num)
 {
   my_gsset_num = gsset_num;
-  last_max_id = (int *)mybmalloc(sizeof(int) * gsset_num);
+  last_max_id = (int *)mymalloc(sizeof(int) * gsset_num);
 #ifdef BEAM
   dimthres_num = hmminfo->opt.vec_size;
-  dimthres = (LOGPROB *)mybmalloc(sizeof(LOGPROB) * dimthres_num);
+  dimthres = (LOGPROB *)mymalloc(sizeof(LOGPROB) * dimthres_num);
 #endif
 }
 
@@ -80,6 +80,19 @@ gms_gprune_prepare()
   }
 }
 
+/**
+ * Free GMS related work area.
+ * 
+ */
+void
+gms_gprune_free()
+{
+  free(last_max_id);
+#ifdef BEAM
+  free(dimthres);
+#endif
+}
+
 /**********************************************************************/
 /* LAST_BEST ... compute the maximum component in last frame first */
 /* BEAM ... use beam pruning */
@@ -108,7 +121,7 @@ calc_contprob_with_safe_pruning(HTK_HMM_Dens *binfo, LOGPROB thres)
   tmp = binfo->gconst;
   for (; veclen > 0; veclen--) {
     x = *(vec++) - *(mean++);
-    tmp += x * x / *(var++);
+    tmp += x * x * *(var++);
     if ( tmp > fthres) {
       return LOG_ZERO;
     }
@@ -143,7 +156,7 @@ calc_contprob_with_beam_pruning_pre(HTK_HMM_Dens *binfo)
   tmp = 0.0;
   for (; veclen > 0; veclen--) {
     x = *(vec++) - *(mean++);
-    tmp += x * x / *(var++);
+    tmp += x * x * *(var++);
     if ( *th < tmp) *th = tmp;
     th++;
   }
@@ -175,7 +188,7 @@ calc_contprob_with_beam_pruning_post(HTK_HMM_Dens *binfo)
   tmp = 0.0;
   for (; veclen > 0; veclen--) {
     x = *(vec++) - *(mean++);
-    tmp += x * x / *(var++);
+    tmp += x * x * *(var++);
     if ( tmp > *(th++)) {
       return LOG_ZERO;
     }
@@ -243,7 +256,7 @@ compute_g_max(HTK_HMM_State *stateinfo, int last_maxi, int *maxi_ret)
     *maxi_ret = maxi;
   }
 
-  return((maxprob + stateinfo->bweight[maxi]) / LOG_TEN);
+  return((maxprob + stateinfo->bweight[maxi]) * INV_LOG_TEN);
 }
   
 #else  /* ~LAST_BEST */
@@ -271,7 +284,7 @@ compute_g_max(HTK_HMM_State *stateinfo)
       maxi = i;
     }
   }
-  return((maxprob + stateinfo->bweight[maxi]) / LOG_TEN);
+  return((maxprob + stateinfo->bweight[maxi]) * INV_LOG_TEN);
 }
 #endif
 
