@@ -38,20 +38,20 @@
  * calc_tied_mix() or calc_mix().
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.6 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
 /* gprune_safe.c --- calculate probability of Gaussian densities */
 /*                   with Gaussian pruning (safe) */
 
-/* $Id: gprune_safe.c,v 1.4 2004/03/22 04:14:32 ri Exp $ */
+/* $Id: gprune_safe.c,v 1.6 2006/12/14 08:18:36 sumomo Exp $ */
 
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
@@ -89,10 +89,10 @@ compute_g_safe(HTK_HMM_Dens *binfo, LOGPROB thres)
   tmp = binfo->gconst;
   for (; veclen > 0; veclen--) {
     x = *(vec++) - *(mean++);
-    tmp += x * x / *(var++);
+    tmp += x * x * *(var++);
     if (tmp > fthres)  return LOG_ZERO;
   }
-  return(tmp / -2.0);
+  return(tmp * -0.5);
 }
 
 
@@ -115,6 +115,18 @@ gprune_safe_init()
   return TRUE;
 }
 
+/**
+ * Free gprune_safe related work area.
+ * 
+ */
+void
+gprune_safe_free()
+{
+  free(OP_calced_score);
+  free(OP_calced_id);
+  free(mixcalced);
+}
+
 /** 
  * @brief  Compute a set of Gaussians with safe pruning.
  *
