@@ -11,13 +11,13 @@
  * @brief  Patricia index tree for name lookup: data type = pointer
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -195,7 +195,7 @@ aptree_add_entry(char *str, void *data, char *matchstr, APATNODE **rootnode)
 
 /*******************************************************************/
 
-static APATNODE *tmproot;
+static APATNODE *tmproot;	///< Work area to hold current root node for removing entry
 
 /** 
  * Recursive sunction to find and remove an entry.
@@ -295,6 +295,7 @@ aptree_traverse_and_do(APATNODE *node, void (*callback)(void *))
 void
 free_aptree(APATNODE *node)
 {
+  if (node == NULL) return;
   if (node->left0 != NULL) free_aptree(node->left0);
   if (node->right1 != NULL) free_aptree(node->right1);
   free(node);
