@@ -18,13 +18,13 @@
  * triphone will be done here.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -57,7 +57,7 @@
  *                 to express the pronunciation
  */
 
-#define PHONEMELEN_STEP  10	///< Memory allocation step for phoneme sequence
+#define PHONEMELEN_STEP  30	///< Memory allocation step for phoneme sequence
 static char buf[MAXLINELEN];	///< Local work area for input text processing
 static char bufbak[MAXLINELEN];	///< Local work area for debug message
 
@@ -385,13 +385,20 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
 {
   char *ptmp, *lp = NULL, *p;
   static char cbuf[MAX_HMMNAME_LEN];
-  int tmpmaxlen, len;
-  HMM_Logical **tmpwseq;
+  static HMM_Logical **tmpwseq = NULL;
+  static int tmpmaxlen;
+  int len;
   HMM_Logical *tmplg;
   boolean pok;
 
   if (strmatch(buf, "DICEND")) return FALSE;
 
+  /* allocate temporal work area for the first call */
+  if (tmpwseq == NULL) {
+    tmpmaxlen = PHONEMELEN_STEP;
+    tmpwseq = (HMM_Logical **)mymalloc(sizeof(HMM_Logical *) * tmpmaxlen);
+  }
+
   /* backup whole line for debug output */
   strcpy(bufbak, buf);
   
@@ -402,14 +409,13 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
     *ok_flag = FALSE;
     return TRUE;
   }
-  winfo->wname[vnum] = strcpy((char *)mymalloc(strlen(ptmp)+1), ptmp);
+  winfo->wname[vnum] = strcpy((char *)mybmalloc2(strlen(ptmp)+1, &(winfo->mroot)), ptmp);
 
   /* just move pointer to next token */
   if ((ptmp = mystrtok_movetonext(NULL, " \t\n")) == NULL) {
     j_printerr("line %d: corrupted data:\n> %s\n", vnum+1, bufbak);
     winfo->errnum++;
     *ok_flag = FALSE;
-    free(winfo->wname[vnum]);
     return TRUE;
   }
 #ifdef CLASS_NGRAM
@@ -433,7 +439,6 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
       j_printerr("line %d: value after '@' missing, maybe wrong space?\n> %s\n", vnum+1, bufbak);
       winfo->errnum++;
       *ok_flag = FALSE;
-      free(winfo->wname[vnum]);
       return TRUE;
     }
     winfo->cprob[vnum] = atof(&(ptmp[1]));
@@ -443,7 +448,6 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
       j_printerr("line %d: corrupted data:\n> %s\n", vnum+1,bufbak);
       winfo->errnum++;
       *ok_flag = FALSE;
-      free(winfo->wname[vnum]);
       return TRUE;
     }
     /* move to the next word entry */
@@ -451,14 +455,12 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
       j_printerr("line %d: corrupted data:\n> %s\n", vnum+1, bufbak);
       winfo->errnum++;
       *ok_flag = FALSE;
-      free(winfo->wname[vnum]);
       return TRUE;
     }
 #else  /* ~CLASS_NGRAM */
     j_printerr("line %d: cannot handle in-class word probability\n> %s\n", vnum+1, ptmp, bufbak);
     winfo->errnum++;
     *ok_flag = FALSE;
-    free(winfo->wname[vnum]);
     return TRUE;
 #endif /* CLASS_NGRAM */
   }
@@ -477,17 +479,15 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
     j_printerr("line %d: missing output string??\n> %s\n", vnum+1, bufbak);
     winfo->errnum++;
     *ok_flag = FALSE;
-    free(winfo->wname[vnum]);
     return TRUE;
   }
   if (ptmp == NULL) {
     j_printerr("line %d: corrupted data:\n> %s\n", vnum+1, bufbak);
     winfo->errnum++;
     *ok_flag = FALSE;
-    free(winfo->wname[vnum]);
     return TRUE;
   }
-  winfo->woutput[vnum] = strcpy((char *)mymalloc(strlen(ptmp)+1), ptmp);
+  winfo->woutput[vnum] = strcpy((char *)mybmalloc2(strlen(ptmp)+1, &(winfo->mroot)), ptmp);
     
   /* phoneme sequence */
   if (hmminfo == NULL) {
@@ -495,8 +495,8 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
     winfo->wseq[vnum] = NULL;
     winfo->wlen[vnum] = 0;
   } else {
-    tmpmaxlen = PHONEMELEN_STEP;
-    tmpwseq = (HMM_Logical **)mymalloc(sizeof(HMM_Logical *) * tmpmaxlen);
+
+    /* store converted phone sequence to temporal bufffer */
     len = 0;
       
     if (do_conv) {
@@ -506,8 +506,6 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
 	j_printerr("line %d: word %s has no phoneme:\n> %s\n", vnum+1, winfo->wname[vnum], bufbak);
 	winfo->errnum++;
 	*ok_flag = FALSE;
-	free(winfo->wname[vnum]);
-	free(winfo->woutput[vnum]);
 	return TRUE;
       }
       cycle_triphone(lp);
@@ -557,6 +555,7 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
 	  tmpmaxlen += PHONEMELEN_STEP;
 	  tmpwseq = (HMM_Logical **)myrealloc(tmpwseq, sizeof(HMM_Logical *) * tmpmaxlen);
 	}
+	/* store to temporal buffer */
 	tmpwseq[len] = tmplg;
       }
       len++;
@@ -565,19 +564,17 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
       j_printerr("> %s\n", bufbak);
       winfo->errnum++;
       *ok_flag = FALSE;
-      free(winfo->wname[vnum]);
-      free(winfo->woutput[vnum]);
       return TRUE;
     }
     if (len == 0) {
       j_printerr("line %d: no phone specified:\n> %s\n", vnum+1, bufbak);
       winfo->errnum++;
       *ok_flag = FALSE;
-      free(winfo->wname[vnum]);
-      free(winfo->woutput[vnum]);
       return TRUE;
     }
-    winfo->wseq[vnum] = tmpwseq;
+    /* store to winfo */
+    winfo->wseq[vnum] = (HMM_Logical **)mybmalloc2(sizeof(HMM_Logical *) * len, &(winfo->mroot));
+    memcpy(winfo->wseq[vnum], tmpwseq, sizeof(HMM_Logical *) * len);
     winfo->wlen[vnum] = len;
   }
   
